﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;

namespace PHPScan
{
    public class HttpResponseReader
    {
        public Stream Stream { get; set; }

        public HttpResponseReader(Stream ResponseStream)
        {
            Stream = ResponseStream;
        }

        public HttpResponse Read()
        {
            var bufferSize = 8192 * 4;
            var response = new HttpResponse();

            var buffer = Stream.Read(bufferSize);

            if (buffer.Length == 0)
                return new HttpResponse() { Header = "", Body = "" };

            var responseString = buffer
                .Select(x => ((char)x).ToString())
                .Aggregate((x, y) => x + y);



            if (!Regex.IsMatch(responseString, @"HTTP/\d\.\d\s\d+\s"))
                return new HttpResponse()
                {
                    Header = "",
                    Body = responseString
                };

            var bodyIndex = responseString.IndexOf("\r\n\r\n");

            if (bodyIndex == -1)
                return new HttpResponse() 
                {
                    Header = responseString,
                    Body = ""
                };
            //var bodyIndex = responseString.IndexOf("\n\n");

            //bodyIndex.Dump();

            //responseString.Dump();

            response.Header = responseString.Remove(bodyIndex);
            response.Body = "";

            var body = responseString.Substring(bodyIndex + 4);

            if (response.Header.ToLower().Contains("\r\ntransfer-encoding: chunked\r\n"))
            {
                BodyChunk chunk = null;

                if (body != "")
                {
                    chunk = BodyChunk.ReadChunk(body, Stream);

                    response.Body += chunk.Body;
                }

                if (chunk == null || !chunk.LastChunk)
                {
                    do
                    {
                        var s = Stream.ReadString(bufferSize);
                        chunk = BodyChunk.ReadChunk(s, Stream);

                        response.Body += chunk.Body;
                    } while (!chunk.LastChunk);
                }
            }
            else
                response.Body = body;

            //response.Dump();



            return response;
        }
    }

}
