﻿using System;
using System.Security.Cryptography;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace PHPScan
{
	public class PHPParser
	{
        public static MD5CryptoServiceProvider md5 = new MD5CryptoServiceProvider();

        private static int _markerIndex;

        static List<string[]> forReplacements = null,
             classReplacements = null;

        private static void ReplaceFors(ref string PHP)
        {
            forReplacements = new List<string[]>();

            var matches = Regex.Matches(PHP, @"for\s*\(", RegexOptions.Multiline);

            foreach (Match m in matches)
            {
                string forCode = Parser.ReadNestedBody(PHP, DelimiterOptions.Parentheses,
                    m.Index, true);

                string digest = md5.ComputeHash(forCode);

                forReplacements.Add(new string[] { digest, forCode });

                PHP = PHP.Replace(forCode, digest);
            }
        }

        private static void RestoreFors(ref string PHP)
        {
            foreach (string[] fors in forReplacements)
                PHP = PHP.Replace(fors[0], fors[1]);

            forReplacements.Clear();
        }



        private static List<PHPFunction> ParseFunctions(string PHP)
        {
            return Parser.ParseCodeBlock<PHPFunction>(PHP, "function", '(');
        }

        private static List<PHPClass> ParseClasses(string PHP)
        {
            var blocks = Parser.ParseCodeBlock<PHPClass>(PHP, "class", '{');

            foreach (PHPClass c in blocks)
                c.Functions = ParseFunctions(c.Body);

            return blocks;
        }

        private static void ReplaceClasses(ref string PHP)
        {
            classReplacements = new List<string[]>();

            var classes = ParseClasses(PHP);

            StringBuilder p = new StringBuilder(PHP);

            foreach (PHPClass c in classes)
            {
                string digest = c.DigestBody();

                p = p.Replace(c.Body, digest);


            }

            forReplacements = new List<string[]>();

            var matches = Regex.Matches(PHP, @"class\s*([^{}]+)\s*{", RegexOptions.Multiline);

            foreach (Match m in matches)
            {
                var forCode = Parser.ReadNestedBody(PHP, DelimiterOptions.Parentheses, m.Index, true);

                var digest = md5.ComputeHash(forCode);

                forReplacements.Add(new string[] { digest, forCode });

                PHP = PHP.Replace(forCode, digest);
            }
        }

        private static string Annotate(string PHP)
        {

            //ParseClasses(PHP);            

            ReplaceFors(ref PHP);

            //ReplaceClasses(ref PHP);

            MatchCollection matches = Regex.Matches(PHP, @";");

            StringBuilder p = new StringBuilder(PHP);

            _markerIndex = 0;

            for (int i = matches.Count - 1; i >= 0; i--)
            {
                var digest = md5.ComputeHash("Marker" + _markerIndex++.ToString());

                var marker = "echo '----" + digest + "----<br />';\r\n";

                p.Insert(matches[i].Index + 1, marker);
            }

            PHP = p.ToString();

            RestoreFors(ref PHP);

            return PHP;
        }
	}
}
