﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace PHPScan
{
    public static class Parser
    {
        const string nameRegex = @"[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*";

        public static string ReadNestedBody(string Code, DelimiterOptions Delimiter, int StartIndex, bool MoveToFirst)
        {
            var d = new Delimiters(Delimiter);            

            if (MoveToFirst)
                StartIndex = Code.IndexOf(d.StartDelimiter, StartIndex);
            else if (Code[StartIndex] != d.StartDelimiter)
                throw new InvalidOperationException("Began on '" + Code[d.StartDelimiter] +
                    "'. Should be '" + d.StartDelimiter + "'");

            StringBuilder body = new StringBuilder();

            int depth = 0;

            for (int x = StartIndex; x < Code.Length; x++)
            {
                body.Append(Code[x]);

                if (Code[x] == d.StartDelimiter)
                    depth++;
                else if (Code[x] == d.EndDelimiter)
                    depth--;

                if (depth == 0)
                    break;
            }

            return body.ToString();
        }

        public static string ReadNestedBody(string Code, DelimiterOptions Delimiter, int StartIndex)
        {
            return ReadNestedBody(Code, Delimiter, StartIndex, false);
        }

        public static void ReplaceDelimiters(ref string Code, char StartDelimiter, char EndDelimiter,
            string NewStartDelimiter, string NewEndDelimiter, int StartIndex, bool MoveToFirst)
        {
            ReplaceDelimiters(ref Code, StartDelimiter, EndDelimiter,
                NewStartDelimiter, NewEndDelimiter, StartIndex, MoveToFirst, true);
        }

        public static void ReplaceDelimiters(ref string Code, char StartDelimiter, char EndDelimiter,
            string NewStartDelimiter, string NewEndDelimiter, int StartIndex, bool MoveToFirst, bool Nested)
        {
            int depth = Nested ? 0 : 1;

            if (MoveToFirst)
            {
                StartIndex = Code.IndexOf(StartDelimiter, StartIndex);
            }
            else if (Code[StartIndex] != StartDelimiter)
                throw new InvalidOperationException("Began on '" + Code[StartDelimiter] +
                    "'. Should be '" + StartDelimiter + "'");

            StringBuilder body = new StringBuilder();
            

            for (int x = StartIndex; x < Code.Length; x++)
            {
                body.Append(Code[x]);

                if (Nested && Code[x] == StartDelimiter)
                    depth++;
                else if (Code[x] == EndDelimiter)
                    depth--;

                if (depth == 0)
                    break;
            }

            string body2 = body.ToString().Remove(0, 1);
            body2 = NewStartDelimiter + body2.Remove(body2.Length - 1, 1) + NewEndDelimiter;

            Code = Code.Remove(StartIndex, body.Length);
            Code = Code.Insert(StartIndex, body2);
        }

        public static List<TCodeBlock> ParseCodeBlock<TCodeBlock>(string Code, string Keyword, char StartDelimiter)
            where TCodeBlock : CodeBlock, new()
        {
            char[] escapeDelimiters = new char[] { '[', '(' };

            var matches = Regex.Matches(Code, Keyword + @"(" + nameRegex + @")\s*" +
                (escapeDelimiters.Contains(StartDelimiter) ? @"\" : "") +
                StartDelimiter, RegexOptions.Multiline);

            List<TCodeBlock> blocks = new List<TCodeBlock>();

            foreach (Match m in matches)
            {
                var b = Parser.ReadNestedBody(Code, DelimiterOptions.Braces, m.Index, true);

                blocks.Add(new TCodeBlock()
                {
                    Name = m.Groups[1].Value,
                    Body = b
                });
            }

            return blocks;
        }
    }
}
