﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;

namespace PHPScan.ScanAlgorithms
{
    public class ArbitraryPHPScanAlgorithm : ScanAlgorithm
    {
        public override int ModeCount
        {
            get { return _codes.Length; }
        }

        private string[] _codes = new string[]
        {
            "<?php echo 'testabc'.'testxyz'; ?>",
            "echo 'testabc'.'testxyz';",
            "<?php echo \"testabc\".\"testxyz\"; ?>",
            "echo \"testabc\".\"testxyz\";",
        };

        private string _server;

        public override string Server
        {
            get { return _server; }
            set { _server = value; }
        }

        public ArbitraryPHPScanAlgorithm(string Server)
        {
            _server = Server;
        }

        public override string BuildRequest(int Mode, string TargetFile, FileTrace SourceTrace)
        {
            //return SourceTrace.Calls.CreateRequest(TargetFile, Server,
            //    _codes[Mode], false, false, true);

            return SourceTrace.Calls.CreateRequest(TargetFile, Server,
                _codes[Mode], false, true, true);
        }

        private ScanAlert CreateAlert(FileTrace TargetTrace)
        {
            return new ScanAlert(ScanAlertOptions.Vulnerability,
                "Arbitrarty PHP Execution", TargetTrace);
        }

        public override ScanAlert ScanTrace(FileTrace TargetTrace)
        {
            if (TargetTrace.Calls
                .Where(x => x.Name == "eval" && x.ParameterValues.Any(y => y.Contains("testabc")))
                .Any())
                return CreateAlert(TargetTrace);

            var falsePositiveRegex = new Regex(@"echo.{0,2}testabctestxyz");

            var Response = falsePositiveRegex.Replace(TargetTrace.Response, "");

            var regex = new Regex(@"testabctestxyz");

            if (regex.IsMatch(Response))
                return CreateAlert(TargetTrace);
            else
                return null;
        }

        public override string ToString()
        {
            return "Arbitrary PHP Execution Scan";
        }
    }
}
