﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace PHPScan.ScanAlgorithms
{
    public class CommandScanAlgorithm : ScanAlgorithm
    {
        public override int ModeCount
        {
            get { return _badStrings.Length; }
        }

        private readonly string[] _badStrings = new string[] 
        {
            //Environment.GetFolderPath(Environment.SpecialFolder.System) +
            //    @"\calc.exe",
            //Environment.GetFolderPath(Environment.SpecialFolder.System) +
            //    "\\calc.exe\x00",

            "calc\x00",
            "|calc\x00",
            //"&&calc\x00",
        };

        private string _server;

        public override string Server
        {
            get { return _server; }
            set { _server = value; }
        }

        public CommandScanAlgorithm(string Server)
        {
            _server = Server;
        }

        public override string BuildRequest(int Mode, string TargetFile, FileTrace SourceTrace)
        {
            return SourceTrace.Calls.CreateRequest(TargetFile, Server,
                _badStrings[Mode], false, false, true);
        }        

        public override ScanAlert ScanTrace(FileTrace TargetTrace)
        {
            var osFunctions = new[]
            {
                "system",
                "exec",
                "shell_exec",
                "passthru"
            };
            if (TargetTrace.Calls
                .Where(x => osFunctions.Contains(x.Name))
                .Any(x => 
                    x.ParameterValues.Any() && 
                    x.ParameterValues[0].Contains("calc")))
            {
                foreach (var c in Process.GetProcessesByName("calc"))
                {
                    c.Kill();
                    c.WaitForExit();
                }

                return new ScanAlert(ScanAlertOptions.Vulnerability,
                    "Command Execution", TargetTrace);
            }

            var processes = Process.GetProcessesByName("calc");
            if (processes.Length != 0)
            {
                foreach (Process p in processes)
                {
                    p.Kill();
                    p.WaitForExit();
                }

                return new ScanAlert(ScanAlertOptions.Vulnerability,
                    "Command Execution", TargetTrace);
            }
            else
                return null;
        }

        public override string ToString()
        {
            return "Command Execution Scan";
        }
    }
}
