﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;

namespace PHPScan.ScanAlgorithms
{
    public class SQLScanAlgorithm : ScanAlgorithm
    {
        private static string[] _anchors = new[] { "testab", "testxy" };

        private readonly string[] _badStrings = new string[]
        {
            //_anchor + @"';--"";--",
            //_anchor + @"\';--\"";--",
            //_anchor + @"\\';--\\"";--",
            //_anchor + @"\\\;--'\\\"";--",
            //_anchor + @"';#"";#",
            //_anchor + @"\';#\"";#",
            //_anchor + @"\\';#\\"";#",
            //_anchor + @"\\\;#'\\\"";#",

            
            _anchors[0] + @"'" + _anchors[1],
            _anchors[0] + @"\'" + _anchors[1],
            _anchors[0] + @"\\'" + _anchors[1],
            _anchors[0] + @"\\\'" + _anchors[1],

            _anchors[0] + @"""" + _anchors[1],
            _anchors[0] + @"\""" + _anchors[1],
            _anchors[0] + @"\\""" + _anchors[1],
            _anchors[0] + @"\\\""" + _anchors[1],
        };

        public override int ModeCount
        {
            get { return _badStrings.Length; }
        }

        private string _server;

        public override string Server
        {
            get { return _server; }
            set { _server = value; }
        }

        public SQLScanAlgorithm(string Server)
        {
            _server = Server;
        }

        public override string BuildRequest(int Mode, string TargetFile, FileTrace SourceTrace)
        {
            return SourceTrace.Calls.CreateRequest(TargetFile, Server,
                _badStrings[Mode], false, true, true);
        }

        private ScanAlert CreateAlert(FileTrace TargetTrace)
        {
            return new ScanAlert(ScanAlertOptions.Vulnerability,
                "SQL Injection", TargetTrace);
        }

        static bool IsSQLInjectable(string SQL)
        {
	        var strings = StringParser.GetStrings(SQL);
        	
	        foreach (var s in strings)
		        SQL = SQL.Replace(s, "");
        		
	        return SQL.Contains(_anchors[0]) || SQL.Contains(_anchors[1]);
        }

        public override ScanAlert ScanTrace(FileTrace TargetTrace)
        {
            var r = TargetTrace.Response.ToLower();

            //if (r.Contains("sql syntax"))
            //    return CreateAlert(TargetTrace);


            //else if (r.Contains("sql"))
            //    return new ScanAlert(ScanAlertOptions.Warning,
            //        "Potential SQL Error", TargetTrace);
            //else
            //    return null;

            foreach (var call in TargetTrace.Calls.Where(x => x.Name == "mysql_query"))
            {
                if (call.ParameterValues.Count == 0)
                    continue;

                var value = call.ParameterValues[0];

                if (IsSQLInjectable(value))
                    return CreateAlert(TargetTrace);

//                var anchorIndex = -1;

//                while (true)
//                {
//                    anchorIndex = value.IndexOf(_anchors[0], anchorIndex + 1);

//                    if (anchorIndex == -1)
//                    {
//#if DEBUG_MESSAGES
//                        Console.WriteLine("Anchor 1 not found");
//#endif

//                        break;
//                    }

//                    var anchor2Index = value.IndexOf(_anchors[1], anchorIndex);

//                    if (anchor2Index == -1)
//                    {
//#if DEBUG_MESSAGES
//                        Console.WriteLine("Anchor 2 not found");
//#endif

//                        break;
//                    }

//                    var testDelimiter = value[anchor2Index - 1];

//                    if (testDelimiter != '\'' &&
//                        testDelimiter != '"')
//                    {
//#if DEBUG_MESSAGES
//                        Console.WriteLine("Delimiter Stripped");
//#endif

//                        continue;
//                    }

//#if DEBUG_MESSAGES
//                    Console.WriteLine("Test Delimiter: {0}", testDelimiter);
//#endif

//                    var delimiters = new[] { '\'', '"', '=' };

//                    for (int i = anchorIndex; i >= 0; i--)
//                    {
//                        if (delimiters.Contains(value[i]))
//                        {
//                            var delimiter = value[i];

//#if DEBUG_MESSAGES
//                            Console.WriteLine("Left Delimiter: {0}", delimiter);
//#endif

//                            if (delimiter == '=')
//                            {
//#if DEBUG_MESSAGES
//                                Console.WriteLine("Equal delimiter -- injection possible");
//#endif

//                                return CreateAlert(TargetTrace);
//                            }

//                            if (testDelimiter != delimiter)
//                            {
//#if DEBUG_MESSAGES
//                                Console.WriteLine("Delimiter mismatch");
//#endif

//                                break;
//                            }

//                            int slashCount = 0;

//                            for (int j = anchor2Index - 2; j >= 0; j--)
//                                if (value[j] == '\\')
//                                    slashCount++;
//                                else
//                                    break;

//#if DEBUG_MESSAGES
//                            Console.WriteLine("Slash count: {0}", slashCount);
//#endif

//                            if (slashCount % 2 == 0)
//                            {
//#if DEBUG_MESSAGES
//                                Console.WriteLine("Even slash count -- injection possible");
//#endif

//                                return CreateAlert(TargetTrace);
//                            }
//#if DEBUG_MESSAGES
//                            else
//                                Console.WriteLine("Uneven slash count");
//#endif

//                            break;
//                        }
//                    }
//                }

//                //Console.Write("");
            }

            return null;
        }

        public override string ToString()
        {
            return "SQL Injection Scan";
        }
    }
}
