﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Text.RegularExpressions;

namespace PHPScan.ScanAlgorithms
{
    class XSSScanAlgorithm : ScanAlgorithm
    {
        private static int _id;

        private string[] _badChars = new string[]
        {
            "<ScRiPt>alert(0)</ScRiPt>",
            "<ScR<script>iPt>alert(0)</ScR<script>iPt>",
            "\"OnMoUsEoVeR=\"alert(0)"
        };

        public override int ModeCount
        {
            get { return 6; }
        }

        private string _server;

        public override string Server
        {
            get { return _server; }
            set { _server = value; }
        }

        public XSSScanAlgorithm(string Server)
        {
            _server = Server;
        }

        public override string BuildRequest(int Mode, string TargetFile, FileTrace SourceTrace)
        {
            //if (Mode == 3 && TargetFile.Contains("attr"))
            //    Console.Write(""); ;
            string chars;
            
            if (Mode < 3)
            {
                chars = _badChars[Mode];
            }
            else
            {
                chars = "\x00" + _badChars[Mode - (ModeCount / 2)];
            }
            
            //return SourceTrace.Calls.CreateRequest(TargetFile, Server,
            //    chars, true, true);
            return SourceTrace.Calls.CreateRequest(TargetFile, Server,
                chars, false, true);
        }

        private bool HasAttributeVulnerability(string Response)
        {
            var index = -1;

            var charsLowered = _badChars[2].ToLower();

            while ((index = Response.IndexOf(charsLowered, index + 1)) != -1)
            {


                for (int i = index; i >= index - 100 && i >= 0; i--)
                {
                    if (Response[i] == '<')
                        return true;
                    else if (Response[i] == '>')
                        break;
                }
            }

            return false;
        }

        public override ScanAlert ScanTrace(FileTrace TargetTrace)
        {
            var respLowered = TargetTrace.Response.ToLower();

            if ((respLowered.Contains(_badChars[0].ToLower()) ||
                HasAttributeVulnerability(respLowered)) &&
                Regex.IsMatch(respLowered, @"^http/\d\.\d\s200\s"))
                return new ScanAlert(ScanAlertOptions.Vulnerability,
                    "Reflected XSS", TargetTrace);            
            else
                return null;
        }

        public override string ToString()
        {
            return "Reflected Cross-site Scripting Scan";
        }
    }
}
