﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using System.ComponentModel;
using PHPScan.ScanAlgorithms;


namespace PHPScan
{
    [Serializable]
    [XmlRoot("Scan")]
    public class ScanConfig : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        private static XmlSerializer _serializer = new XmlSerializer(typeof(ScanConfig));

        private bool _BeepOnAlert = false;
        
        public bool BeepOnAlert
        {
        	get { return _BeepOnAlert; }
        	set
        	{                
        		_BeepOnAlert = value;
        
        		RaisePropertyChanged("BeepOnAlert");
        	}
        }

        private bool _unhook = true;

        [XmlAttribute]
        public bool Unhook
        {
            get { return _unhook; }
            set { _unhook = value; RaisePropertyChanged("Unhook"); }
        }

        private bool _runViewer = false;

        [XmlAttribute]
        public bool RunViewer
        {
            get { return _runViewer; }
            set { _runViewer = value; RaisePropertyChanged("RunViewer"); }
        }

        private bool _DumpMessages;
        
        [XmlAttribute]
        public bool DumpMessages
        {
        	get { return _DumpMessages; }
        	set
        	{                
        		_DumpMessages = value;
        
        		RaisePropertyChanged("DumpMessages");
        	}
        }

        //private bool _LauncherUsed;
        
        //[XmlAttribute]
        //public bool LauncherUsed
        //{
        //    get { return _LauncherUsed; }
        //    set
        //    {                
        //        _LauncherUsed = value;

        //        RaisePropertyChanged("LauncherUsed");
        //    }
        //}

        private bool _arbitraryPHPScan = true;

        [XmlAttribute("ArbitraryPHP")]
        public bool ArbitraryPHPScan
        {
            get { return _arbitraryPHPScan; }
            set { _arbitraryPHPScan = value; RaisePropertyChanged("ArbitraryPHPScan"); }
        }

        private bool _commandScan = true;

        [XmlAttribute("Command")]
        public bool CommandScan
        {
            get { return _commandScan; }
            set { _commandScan = value; RaisePropertyChanged("CommandScan"); }
        }

        private bool _fileScan = true;

        [XmlAttribute("File")]
        public bool FileScan
        {
            get { return _fileScan; }
            set { _fileScan = value; RaisePropertyChanged("FileScan"); }
        }

        private bool _lfiScan = true;

        [XmlAttribute("LFI")]
        public bool LFIScan
        {
            get { return _lfiScan; }
            set { _lfiScan = value; RaisePropertyChanged("LFIScan"); }
        }

        private bool _sqlScan = true;

        [XmlAttribute("SQL")]
        public bool SQLScan
        {
            get { return _sqlScan; }
            set { _sqlScan = value; RaisePropertyChanged("SQLScan"); }
        }

        private bool _xssScan = true;

        [XmlAttribute("XSS")]
        public bool XSSScan
        {
            get { return _xssScan; }
            set { _xssScan = value; RaisePropertyChanged("XSSScan"); }
        }

        private bool _openRedirect = true;

        [XmlAttribute]
        public bool OpenRedirect
        {
            get { return _openRedirect; }
            set { _openRedirect = value; RaisePropertyChanged("OpenRedirect"); }
        }

        private bool _pathDisclosure = true;

        [XmlAttribute]
        public bool PathDisclosure
        {
            get { return _pathDisclosure; }
            set { _pathDisclosure = value; RaisePropertyChanged("PathDisclosure"); }
        }

        private string _server = "localhost";

        [XmlAttribute]
        public string Server
        {
            get { return _server; }
            set { _server = value; RaisePropertyChanged("Server"); }
        }

        private string _webRoot;

        [XmlAttribute]
        public string WebRoot
        {
            get { return _webRoot; }
            set { _webRoot = value; RaisePropertyChanged("WebRoot"); }
        }

        private bool _discoveryReport;

        [XmlAttribute]
        public bool DiscoveryReport
        {
            get { return _discoveryReport; }
            set { _discoveryReport = value; RaisePropertyChanged("DiscoveryReport"); }
        }

        private string[] _applicationPaths;

        [XmlElement("ApplicationPath")]
        public string[] ApplicationPaths
        {
            get { return _applicationPaths; }
            set { _applicationPaths = value; RaisePropertyChanged("ApplicationPaths"); }
        }

        private string _appPathsString;

        [XmlAttribute]
        public string AppPathsString
        {
            get { return _appPathsString; }
            set { _appPathsString = value; RaisePropertyChanged("AppPathsString"); }
        }

        private bool _allApps = true;

        [XmlAttribute]
        public bool AllApps
        {
            get { return _allApps; }
            set { _allApps = value; RaisePropertyChanged("AllApps"); }
        }

        private List<ScanAlgorithm> _scanAlgorithms = new List<ScanAlgorithm>();

        [XmlIgnore]
        public List<ScanAlgorithm> ScanAlgorithms
        {
            get { return _scanAlgorithms; }
        }

        public void Serialize(string File)
        {
            _serializer.Serialize(File, this);
        }

        private void RaisePropertyChanged(string Property)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(Property));
        }

        public string[] Validate()
        {
            var errors = new List<string>();

            if (_webRoot == null)
                errors.Add("No webroot specified");
            else if (!new DirectoryInfo(_webRoot).Exists)
                errors.Add(string.Format("Could not find webroot {0}",
                    _webRoot));
            else if (!AllApps)
            {
                if (string.IsNullOrEmpty(_appPathsString))
                    errors.Add("No applications paths specified. Use * to target every " +
                        "application in the webroot");
                else
                {
                    foreach (var appPath in _appPathsString
                        .Split(',')
                        .Select(x => _webRoot + "\\" + x.Trim('"')))
                        if (!new DirectoryInfo(appPath).Exists)
                            errors.Add(string.Format("Could not find application path {0}",
                                appPath));
                }
            }

            return errors.ToArray();
        }

        public string GetArgs()
        {
            var args = string.Format("-s {0} ", Server);

            args += "-m ";

            if (CommandScan)
                args += "C";

            if (FileScan)
                args += "F";

            if (LFIScan)
                args += "L";

            if (ArbitraryPHPScan)
                args += "P";

            if (SQLScan)
                args += "S";

            if (XSSScan)
                args += "X";

            if (OpenRedirect)
                args += "R";

            if (PathDisclosure)
                args += "I";

            args += " ";

            if (DiscoveryReport)
                args += "-d ";

            if (BeepOnAlert)
                args += "-b ";

            if (DumpMessages)
                args += "-dump ";

            if (RunViewer)
                args += "-v ";

            args += "\"" + WebRoot + "\" " + (AllApps ? "*" : AppPathsString);

            return args;
        }

        public static ScanConfig Deserialize(string File)
        {
            return _serializer.Deserialize(File) as ScanConfig;
        }

        public static ScanConfig Create(string[] args)
        {
            var config = new ScanConfig();

            int argIndex = 0;

            for (int i = 0; i < args.Length; i++)
            {
                if (args[i] == "-s")
                {
                    config.Server = args[i + 1];
                    i++;

                    foreach (var a in config.ScanAlgorithms)
                        a.Server = config.Server;
                }
                //else if (args[i] == "-l")
                //{
                //    config.LauncherUsed = true;
                //}
                else if (args[i] == "-n")
                {
                    config.Unhook = false;
                }
                else if (args[i] == "-v")
                {
                    config.RunViewer = true;
                }
                else if (args[i] == "-d")
                {
                    config.DiscoveryReport = true;
                }

                else if (args[i] == "-dump")
                {
                    config.DumpMessages = true;
                }
                else if (args[i] == "-b")
                {
                    config.BeepOnAlert = true;
                }
                else if (args[i] == "-l")
                {
                    // Nothing
                }
                else if (args[i] == "-m")
                {
                    var modes = args[i + 1];

                    i++;

                    foreach (var c in modes)
                    {
                        ScanAlgorithm scan = null;

                        switch (c.ToString().ToLower()[0])
                        {
                            case 'c':
                                scan = new CommandScanAlgorithm(config.Server);
                                break;
                            case 'l':
                                try
                                {
                                    scan = new LocalFileInclusionScanAlgorithm(config.Server);
                                }
                                catch (UnauthorizedAccessException ex)
                                {
                                    UI.DisplayCriticalMessageAndExit("Error writing LFI test file. Ensure that " +
                                        "PHP Vulnerability Hunter has administrative privileges.");
                                }
                                break;
                            case 'f':
                                scan = new FileScanAlgorithm(config.Server);
                                break;
                            case 'p':
                                scan = new ArbitraryPHPScanAlgorithm(config.Server);
                                break;
                            case 's':
                                scan = new SQLScanAlgorithm(config.Server);
                                break;
                            case 'x':
                                scan = new XSSScanAlgorithm(config.Server);
                                break;
                            case 'i':
                                scan = new FullPathDisclosureAlgorithm(config.Server);
                                break;
                            case 'r':
                                scan = new OpenRedirectScanAlgorithm(config.Server);
                                break;
                        }

                        if (scan == null)
                            UI.DisplayCriticalMessageAndExit("Invalid scan mode: " + c);

                        config.ScanAlgorithms.Add(scan);
                    }
                }
                else
                {
                    switch (argIndex)
                    {
                        case 0:
                            config.WebRoot = args[i];
                            break;
                        case 1:
                            if (args[i] == "*")
                            {
                                var dir = new DirectoryInfo(config.WebRoot);
                                config.ApplicationPaths = dir.GetDirectories()
                                    .Select(x => x.Name)
                                    .ToArray();
                            }
                            else
                                config.ApplicationPaths = args[i].Split(',');
                            break;
                    }

                    argIndex++;
                }
            }

            if (argIndex != 2)
                UI.DisplayCriticalMessageAndExit("Invalid argument count");

            // Validate user input

            if (!Directory.Exists(config.WebRoot))
                UI.DisplayCriticalMessageAndExit("Web root {0} not found.", config.WebRoot);

            if (config.ScanAlgorithms.Count == 0)
            {
                LocalFileInclusionScanAlgorithm lfi = null;

                try
                {
                    lfi = new LocalFileInclusionScanAlgorithm(config.Server);
                }
                catch (UnauthorizedAccessException ex)
                {
                    UI.DisplayCriticalMessageAndExit("Error writing LFI test file. Ensure that " +
                        "PHP Vulnerability Hunter has administrative privileges.");
                }

                config._scanAlgorithms = new List<ScanAlgorithm>()
                {
                    new CommandScanAlgorithm(config.Server),
                    new FileScanAlgorithm(config.Server),
                    lfi,                    
                    new ArbitraryPHPScanAlgorithm(config.Server),
                    new SQLScanAlgorithm(config.Server),                    
                    new XSSScanAlgorithm(config.Server),
                    new OpenRedirectScanAlgorithm(config.Server),
                    new FullPathDisclosureAlgorithm(config.Server),
                };
            }

            return config;
        }        
    }
}
