﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;

namespace PHPScan
{
    public class ScanMetrics
    {
        public static ScanMetrics Default = new ScanMetrics();

        public FileInfo AnnotationFile { get; set; }

        private Dictionary<string, int> _annotationTable = new Dictionary<string, int>();

        public Dictionary<string, int> AnnotationTable
        {
            get { return _annotationTable; }
        }

        public string AnnotateCode(string File, string Code)
        {
            //File = Regex.Replace(File, @":|\\|/", "_");
            var x = 0;
            var matches = Regex
                .Matches(Code, @"/fern\.php'\);")
                .OfType<Match>()
                .Concat(Regex
                    .Matches(Code, @"function\s+" + PHP.ValidNameRegex + @"\([^)]*\)\s*{")
                    .OfType<Match>())
                .Reverse();

            foreach (var match in matches)
            {
                Code = Code.Insert(match.Index + match.Length,
                    string.Format("\r\nAnnotation(\"{0}_{1}\");\r\n",
                        File.Replace("\\", "\\\\"), x++));
            }

            _annotationTable.Add(File, x);

            return Code;
        }

        public void Reset()
        {
            AnnotationFile.Delete();
        }

        public CodeCoverageTable CalculateCoverage()
        {
            if (!AnnotationFile.Exists)
                return new CodeCoverageTable();

            var a = File.ReadAllText(AnnotationFile.FullName);

            var entries = a
                .Split(new [] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
                .Distinct();

            var entryTable = new CodeCoverageTable();

            foreach (var entry in entries)
            {
                var e = entry.RemoveAtLastIndexOf('_');

                if (entryTable.ContainsKey(e))
                    entryTable[e]++;
                else
                    entryTable.Add(e, 1);
            }

            var total = 
                entryTable.Sum(x => x.Value) / AnnotationTable.Sum(x => x.Value) * 100;

            foreach (var key in AnnotationTable.Keys)
            {
                if (entryTable.ContainsKey(key))
                    entryTable[key] = entryTable[key] / AnnotationTable[key] * 100;
                else
                    entryTable.Add(key, AnnotationTable[key] == 0 ? 100 : 0);
            }

            entryTable.Add("Total", total);

            return entryTable;
        }
    }
}
