﻿using System;
using System.Linq;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Xml.Serialization;
using System.Diagnostics;
using PHPScan;
using System.Threading;
using System.Reflection;

namespace PHPVHGUI
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window, INotifyPropertyChanged
    {
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        string _configFile = new FileInfo(System.Reflection.Assembly.GetExecutingAssembly().Location).DirectoryName + 
            "\\config.xml";

        string _phpvhExe;

        private ScanConfig _config = new ScanConfig() { RunViewer = true };

        public ScanConfig Config
        {
            get { return _config; }
        }

        private System.Windows.Forms.FolderBrowserDialog _folderDialog = new System.Windows.Forms.FolderBrowserDialog();

        public Window1()
        {
            InitializeComponent();

            Loaded += new RoutedEventHandler(Window1_Loaded);
            Closing += new CancelEventHandler(Window1_Closing);

            var l = Assembly.GetExecutingAssembly().Location;

            _phpvhExe = new FileInfo(l).Directory + "\\phpvh.exe";

            DataContext = this;           

        }

        private void SaveConfig()
        {
            _config.Serialize(_configFile);
        }

        void Window1_Loaded(object sender, RoutedEventArgs e)
        {
            if (File.Exists(_configFile))
            {
                _config = ScanConfig.Deserialize(_configFile);

                PropertyChanged(this, new PropertyChangedEventArgs("Config"));
            }

            SetAppsEnabled();
        }

        void Window1_Closing(object sender, CancelEventArgs e)
        {
            SaveConfig();
        }

        private void ShowDialog(string Message, string Title)
        {
            new DialogWindow()
            {
                Message = Message,
                Owner = this,
                Title = Title,
                WindowStartupLocation = WindowStartupLocation.CenterOwner
            }.ShowDialog();
        }

        private bool Validate()
        {
            var messages = _config.Validate();

            if (!File.Exists(_phpvhExe))
                messages.Add("phpvh.exe not found");

            if (messages.Count == 0)
                return true;

            ShowDialog(messages.Aggregate((x, y) => x + "\r\n\r\n" + y),
                "Validation Errors");

            return false;
        }

        private void StartButton_Click(object sender, RoutedEventArgs e)
        {
            SaveConfig();

            if (!Validate())
                return;            

            var args = _config.GetArgs() + " -l";

            
            

            var process = new Process() 
            { 
                StartInfo = new ProcessStartInfo()
                {
                    Verb = "runas",
                    FileName = _phpvhExe,
                    Arguments = args,
                    //UseShellExecute = false,
                    //ErrorDialog = false,
                    
                    //RedirectStandardInput = true,
                    //RedirectStandardOutput = true,
                    //RedirectStandardError = true, 
                    //WindowStyle = ProcessWindowStyle.Hidden,
                    //CreateNoWindow = true,
                },
                //EnableRaisingEvents = true,
            };

            //ConsoleText.Text = "";

            //process.OutputDataReceived += (o, _e) =>
            //    {
            //        ConsoleText.Dispatcher.Invoke((Action)(() =>
            //        {
            //            ConsoleText.Text += _e.Data + "\r\n";
            //            ConsoleText.CaretIndex = ConsoleText.Text.Length;
            //            ConsoleText.ScrollToEnd();
            //        }));
            //    };


            try
            {
                process.Start();
            }
            catch (Win32Exception ex)
            {
                MessageBox.Show("Error starting phpvh\r\n\r\n" + ex.Message, 
                    "Error starting phpvh", MessageBoxButton.OK, MessageBoxImage.Error);

                return;
            }
            //process.BeginOutputReadLine();

            //var writer = process.StandardInput;
            //var reader = process.StandardOutput;
            //var errorReader = process.StandardError;

            //char[] buffer = null;

            

            //ThreadPool.QueueUserWorkItem(x =>
            //{
            //    while (!process.HasExited)
            //    {
            //        buffer = new char[8192 * 4];
            //        var len = reader.Read(buffer, 0, buffer.Length);

            //        var s = new string(buffer, 0, len);

            //        ConsoleText.Dispatcher.Invoke((Action)(() =>
            //        {
            //            ConsoleText.Text += s;
            //            ConsoleText.CaretIndex = ConsoleText.Text.Length;
            //            ConsoleText.ScrollToEnd();
            //        }), System.Windows.Threading.DispatcherPriority.Send, null);

            //        Thread.Sleep(500);
            //    }
            //});

            Environment.Exit(0);
        }

        private void WebrootBrowseButton_Click(object sender, RoutedEventArgs e)
        {
            if (Directory.Exists(_config.WebRoot))
                _folderDialog.SelectedPath = _config.WebRoot;

            if (_folderDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                _config.WebRoot = _folderDialog.SelectedPath;                
            }
        }

        private void AppsButton_Click(object sender, RoutedEventArgs e)
        {

        }

        private void SetAppsEnabled()
        {
            AppsTextBox.IsEnabled = !_config.AllApps;
        }

        private void ScanAllCheckBox_Click(object sender, RoutedEventArgs e)
        {
            SetAppsEnabled();
        }

        private void ExitMenuItem_Click(object sender, RoutedEventArgs e)
        {
            SaveConfig();

            Environment.Exit(0);
        }

        private void HelpMenuItem_Click(object sender, RoutedEventArgs e)
        {
            Process.Start("http://www.autosectools.com/Page/PHP-Vulnerability-Hunter-Guide");
        }
    }
}
