﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml.Serialization;

namespace PHPScan
{
    [Serializable]
    public class FileTrace
    {
        private string _file;
        
        [XmlAttribute]
        public string File
        {
            get { return _file; }
            set { _file = value; }
        }

        private string _request;

        [XmlElement]
        public string Request
        {
            get { return _request; }
            set { _request = value; }
        }

        private string _response;

        [XmlElement]
        public string Response
        {
            get { return _response; }
            set { _response = value; }
        }

        private string _rawText;

        [XmlElement]
        public string RawText
        {
            get { return _rawText; }
            set { _rawText = value; }
        }

        private List<FunctionCall> _calls = new List<FunctionCall>();

        [XmlElement]
        public List<FunctionCall> Calls
        {
            get { return _calls; }
            set { _calls = value; }
        }

        public override string ToString()
        {
            var s = new StringBuilder();

            var callDictionary = new Dictionary<string,Dictionary<string, string>>();
            
            foreach (var call in _calls)
            {
                if (!callDictionary.ContainsKey(call.Name))
                    callDictionary.Add(call.Name, new Dictionary<string, string>());

                if (call.ParameterValues.Any() &&
                    !callDictionary[call.Name].ContainsKey(call.ParameterValues[0]))
                    callDictionary[call.Name].Add(call.ParameterValues[0], call.Value);
            }

            foreach (var call in callDictionary)
            {
                s.Append(call.Key);
                s.AppendLine();

                foreach (var callParam in call.Value)
                {
                    s.Append(callParam.Key.Indent() + " = " + callParam.Value);
                    s.AppendLine();
                }

                s.AppendLine();
            }

            return s.ToString();
        }
    }
}
